/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */

#ifndef headerfilecmslucorebbcodeparser
#define headerfilecmslucorebbcodeparser

#include "htmlfilter.h"


class BBCODEParser : public HTMLFilter
{

	//using HTMLFilter::pchar;


	struct Tags
	{
/*
		const wchar_t * bbcode;
		const wchar_t * html_tag;
		const wchar_t * html_arg_prefix;
		const wchar_t * html_arg_postfix;
		const wchar_t * additional_html_tag_prefix;
		const wchar_t * additional_html_tag_postfix;
		bool inline_tag;
*/
		const wchar_t * bbcode;
		const wchar_t * html_tag;
		const wchar_t * html_argument; // with closing '>'
		bool inline_tag;
	};

	bool Equal(const wchar_t * str1, const wchar_t * str2);

	virtual bool IsValidCharForName(int c);

	virtual bool IsOpeningTagMark();
	virtual bool IsOpeningCommentaryTagMark();
	virtual bool SkipCommentaryTagIfExists();
	virtual bool IsClosingTagMark();
	virtual bool IsClosingXmlSimpleTagMark();


	void PutHtmlArgument1(const wchar_t * arg_start, const wchar_t * arg_end, bool has_u);
	void PutHtmlArgument2(const Tags * tag, bool has_u);
	void PutHtmlArgument(const Tags * tag, const wchar_t * arg_start, const wchar_t * arg_end);

	void PutOpeningTagFromEzc(const wchar_t * start, const wchar_t * end);
	void PutOpeningTagFromBBCode(const Tags * tag, const wchar_t * start, const wchar_t * end);

	virtual void PutOpeningTag(const wchar_t * start, const wchar_t * end);
	virtual void PutClosingTag(const wchar_t * tag);

	const Tags * FindTag(const wchar_t * tag);
	void PrintArgumentCheckQuotes(const wchar_t * & start, const wchar_t * & end);


	void PrintEscape(int c, bool change_quote = false);
	void PrintEncode(int c);

	void PrintEscape(const wchar_t * start, const wchar_t * end, bool change_quote = false);
	void PrintEncode(const wchar_t * start, const wchar_t * end);

	void PrintArgumentEncode(const wchar_t * start, const wchar_t * end);
	void PrintArgumentEscape(const wchar_t * start, const wchar_t * end);

	virtual void ReadNormalTextSkipWhite(const wchar_t * & start, const wchar_t * & last_non_white);
	virtual void PutNormalText(const wchar_t * str, const wchar_t * end);
	virtual void PutNormalTextTrim(const wchar_t * str, const wchar_t * end);

	virtual void CheckExceptions();

	virtual void Init();
	virtual void Deinit();

	void PutClosingTag(const Tags * tag);


	void CheckOpeningTag(const Tags * tag, const wchar_t * tag_name, bool & condition);
	void CheckOpeningTag(const Tags * tag);

	void TrimWhiteWithNewLines(const wchar_t * & start, const wchar_t * & end);

	bool has_open_ol_tag; // has open html <ol> tag
	bool has_open_ul_tag; // has open html <ul> tag
	bool has_open_li_tag; // has open html <li> tag


};

#endif
