/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfilecmslucorelog
#define headerfilecmslucorelog


#include <sstream>
#include <fstream>
#include <iostream>
#include <string>
#include "textstream.h"


// log1 - the first level
// log2
// log3
// log4 - the last level (debug level)
// logend        - the end of a line
// logendrequest - end of a current request
// logsave       - current log buffer is saved and cleared
enum Manipulators { log1, log2, log3, log4, logend, logendrequest, logsave };


class Log
{
public:

	Log();
	~Log();

	void Init(int log_level_, bool save_each_line_, const std::string & log_file_, bool log_std, int log_max_requests);

	void PutDate(Manipulators m);
	Log & operator<<(const void * s);
	Log & operator<<(const char * s);
	Log & operator<<(const std::string * s);
	Log & operator<<(const std::string & s);
	Log & operator<<(const wchar_t * s);
	Log & operator<<(const std::wstring * s);
	Log & operator<<(const std::wstring & s);
	Log & operator<<(int s);
	Log & operator<<(long s);
	Log & operator<<(char s);
	Log & operator<<(size_t s);
	Log & operator<<(double s);
	Log & operator<<(Manipulators m);
	
	void SystemErr(int err);
	void SaveLog();
	void SaveLogAndClear();

private:

	// buffer for the log
	TextStream<std::wstring> buffer;

	// log lovel from the config file
	int log_level;

	// current level set by a modifier (e.g. log << log3)
	int current_level;

	// current request for logging
	// starts from zero and incremented after logendrequest modifier
	int request;
	
	// how many request to save at once
	int max_requests;

	// file log
	std::string log_file;
	std::ofstream file;

	// logging to stdout
	bool log_stdout;

	// how many lines there are in the buffer
	int lines;

	// is the config file already open
	bool log_file_open;

	// how many lines can be in the config buffer
	// default: 5000
	int max_lines;

	// whether to save each line (for debug)
	bool save_each_line;

	void OpenFile();

};



extern Log log;
extern Log nlog;



#endif

