/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2009-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */

#include "mount.h"
#include "misc.h"



Mount::Mount()
{
	dir_id = -1;
	type   = -1;
	fs     = -1;
}



void Mount::ClearParams()
{
size_t i;

	for(i=0 ; i<param.size() ; ++i)
		param[i].Clear();
}


bool Mount::IsPar(int code)
{
	if( code < 0 || code >= (int)param.size() )
		return false;

	if( !param[code].defined )
		return false;

return true;
}



bool Mount::IsArg(int code, const wchar_t * arg)
{
	ParamRow::ParamArg::iterator i;	

	if( code < 0 || code >= (int)param.size() )
		return false;

	if( !param[code].defined )
		return false;

	for(i=param[code].arg.begin() ; i!=param[code].arg.end() ; ++i)
	{
		if( *i == arg )
			return true;
	}

return false;
}


bool Mount::IsArg(int code, const std::wstring & arg)
{
	return IsArg(code, arg.c_str());
}


bool Mount::IsArg(int code, int arg)
{
	ParamRow::ParamArg::iterator i;	

	if( code < 0 || code >= (int)param.size() )
		return false;

	if( !param[code].defined )
		return false;

	for(i=param[code].arg.begin() ; i!=param[code].arg.end() ; ++i)
	{
		if( Toi(*i) == arg )
			return true;
	}

return false;
}


const std::wstring & Mount::Arg(int code, int arg) const
{
	if( code < 0 || code >= (int)param.size() )
		return empty_str;

	if( !param[code].defined )
		return empty_str;

	if( arg >= (int)param[code].arg.size() )
		return empty_str;

return param[code].arg[arg];
}


const std::wstring & Mount::FirstArg(int code) const
{
	return Arg(code, 0);
}

