/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "createticket.h"
#include "functions/functions.h"
#include "sessiondata.h"



namespace Ticket
{



CreateTicket::CreateTicket()
{
	fun.url = L"createticket";
}


void CreateTicket::SetTDb(TDb * ptdb)
{
	tdb = ptdb;
}


void CreateTicket::SetTicketInfo(TicketInfo * pinfo)
{
	ticket_info = pinfo;
}




bool CreateTicket::HasAccess()
{
	if( request->is_item || !system->HasWriteAccess(*request->dir_tab.back()) )
		return false;
	
	if( !system->mounts.pmount || system->mounts.pmount->type != ticket_info->mount_type_ticket )
		return false;

return true;
}



void CreateTicket::AddTicket(Ticket & ticket, Item & item)
{
	ticket.file_id  = item.id;
	request->status = tdb->AddTicket(ticket);

	if( request->status == WINIX_ERR_OK )
	{
		// sending notification
		notify_msg.code           = WINIX_NOTIFY_CODE_USER1;
		notify_msg.template_index = ticket_info->template_index;
		notify_msg.dir_link       = config->base_url;
		system->dirs.MakePath(item.parent_id, notify_msg.dir_link, false);
		notify_msg.item_link      = notify_msg.dir_link;
		notify_msg.item_link     += item.url;

		system->notify.ItemChanged(notify_msg);
	}
}




void CreateTicket::Submit(Ticket & ticket, Item & item)
{
	if( functions->CheckAbuse() )
		return;

	functions->SetUser(item);
	item.privileges = 0644; // !! tymczasowo
	item.parent_id  = request->dir_tab.back()->id;
		
	// adding without notificating
	request->status = system->AddFile(item, false);

	if( request->status == WINIX_ERR_OK )
		AddTicket(ticket, item);
	
	if( request->status == WINIX_ERR_OK )
	{
		log << log2 << "CreateTicket: added a new ticket" << logend;
		RemoveTicket();
		system->RedirectTo(item);
	}
	else
	{
		log << log1 << "CreateTicket: problem with adding a new ticket, error code: "
			<< request->status << logend;
	}
}



void CreateTicket::RemoveTicket()
{
	SessionData * session_data = reinterpret_cast<SessionData*>(
								 request->session->plugin_data.Get(ticket_info->plugin_id) );

	long dir_id = request->dir_tab.back()->id;
	session_data->create_ticket_map.erase(dir_id);
}



Ticket & CreateTicket::PrepareTicket()
{
	SessionData * session_data = reinterpret_cast<SessionData*>(
								 request->session->plugin_data.Get(ticket_info->plugin_id) );

	long dir_id     = request->dir_tab.back()->id;
	Ticket & ticket = session_data->GetTicket(dir_id, session_data->create_ticket_map);

return ticket;
}



void CreateTicket::MakePost()
{
	ticket_info->Clear();
	ticket_info->FindCurrentConf();

	Ticket & ticket = PrepareTicket();
	Item   & item   = request->item;
	ticket_info->ticket = &ticket;

	ticket_info->ReadTicketParams(ticket, false);
	functions->ReadItem(item, Item::file);

	if( !request->IsPostVar(L"fileuploadsubmit") )
		Submit(ticket, item);
}



void CreateTicket::MakeGet()
{
	ticket_info->Clear();
	ticket_info->FindCurrentConf();

	Ticket & ticket = PrepareTicket();
	ticket_info->ticket = &ticket;
}




} // namespace
