/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "templates.h"
#include "../core/request.h"
#include "../core/misc.h"




namespace TemplatesFunctions
{



void user_logged(Info & i)
{
	i.res = request->session->puser != 0;
}


void user_name(Info & i)
{
	if( !request->session->puser )
		return;
	
	i.out << request->session->puser->name;
}


// should be at least in one group
// !! moze lepsza nazwa?
void user_is_in_group(Info & i)
{
	if( !request->session->puser || i.params.empty() )
		return;

	for(size_t a=0 ; a<i.params.size() ; ++a)
	{
		long gid = system->groups.GetGroupId(i.params[a]);

		if( gid!=-1 && request->session->puser->IsMemberOf(gid) )
		{
			i.res = true;
			break;
		}
	}
}


// !! moze lepsza nazwa?
void user_is_in_all_groups(Info & i)
{
	if( !request->session->puser || i.params.empty() )
		return;

	for(size_t a=0 ; a<i.params.size() ; ++a)
	{
		long gid = system->groups.GetGroupId(i.params[a]);

		if( gid==-1 || !request->session->puser->IsMemberOf(gid) )
			return;
	}

	i.res = true;
}


void user_super_user(Info & i)
{
	if( request->session->puser && request->session->puser->super_user )
		i.res = true;
}


void user_can_use_html(Info & i)
{
	if( !request->session->puser )
		i.res = false;
	else
		i.res = system->CanUseHtml(request->session->puser->id);
}


void user_can_use_bbcode(Info & i)
{
	if( !request->session->puser )
		i.res = false;
	else
		i.res = system->CanUseBBCode(request->session->puser->id);
}


void user_can_use_raw(Info & i)
{
	if( !request->session->puser )
		i.res = false;
	else
		i.res = system->CanUseRaw(request->session->puser->id);
}



} // namespace TemplatesFunctions



